/**
 * ==================================================================
 *
 * Program Name
 *  MRTR - Message Router
 *
 * Last date of modification
 *  February 14, 2011
 *
 * Description
 *  This program will get a message from a queue and output it
 *  to one or more queues.  Context information is maintained
 *  across the message put(s).  Basically, this program is a
 *  message multiplexer.
 *
 *  The following is the logic in this program:
 *
 *  - Initialize and allocated memory.
 *  - Get the command line parameters.
 *  - Read the ini file.
 *  - Connect to the queue manager.
 *  - Open the input queue
 *  - Read the first Message under syncpoint
 *  - While more messages
 *  -   Find tag in the message and lookup value in the inifile.
 *  -   PUT1
 *  -   Commit messages
 *  -   Read the next message under syncpoint
 *  - EndWhile
 *  - Close the input queue
 *  - Disconnect from the queue manager.
 *  - Output statistical information.
 *  - Terminate
 *
 * @copyright Copyright(C), 1996-2011, Capitalware Inc.
 * @author Roger Lacroix, Capitalware Inc.
 * @return 0 for ok or -1 for error
 * @version 1.1.0
 * @license Apache 2 License
 *
 * ------------------------------------------------------------------
 */
#if defined(__MVS__)
#define MVS /* define our MVS indicator */
#define _XOPEN_SOURCE_EXTENDED 1
#endif

/**
 * Program build time eye-catchers for executable.
 */
#define PROGRAM_NAME "MRTR"
static const char * EYECATCHER_EyeCatcher  = PROGRAM_NAME "  ";
static const char * EYECATCHER_Version     = "1.1.0" ;
static const char * EYECATCHER_Datestamp   = __DATE__ ;
static const char * EYECATCHER_Timestamp   = __TIME__ ;
static const char * EYECATCHER_Filename    = __FILE__ ;

/**
 * Include the C standard header files
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

/**
 * Include MQSeries header files
 */
#include <cmqc.h>

/**
 * Message Router include file.
 */

#if defined(MVS) || defined(OS400)
#include "mrtr.h"
#else
#include "hdrs/mrtr.h"
#endif


/**
 * Global Storage.
 */

char* usage = "-i ini_filename                                     \n"\
"-i ini_filename    The name of the ini file                       \n"\
"-?                 Display this help message            [Optional]\n";

/**
 * MRTR's global work storage.
 */
MRTR_GLOBAL_T  MR;


/**
 * Mainline.
 */
int main( int argc, char **argv )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int          rc=0;
   int          rc2=0;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   rc = Initialize( );

   if ( rc == 0 )
   {
      /* Get the command line parameters */
      rc = GetCmdLineParms( argc, argv );
   }

   if ( rc == 0 )
   {
      /* Read ini file */
     rc = GetInitFileParameters( );
   }


   if ( rc == 0 )
   {
      /* Connect to the local queue manager */
      rc = ConnectToMqm( );
   }

   if ( rc == 0 )
   {
      /* Open the Input Q */
      rc = OpenInputQueue( );
   }

   if ( rc == 0 )
   {
      /* Loop the messages */
      rc = ProcessMessages( );
   }

   if ( MR.OpenedInputQueue == 1 )
   {
      /* We're done - close the Input Q */
      rc2 = CloseInputQueue( );

      if ( rc2 > rc )
      {
         rc = rc2;
      }
   }

   /* if we are connected then disconnect */

   if ( MR.ConnectedFlag == 1 )
   {
      rc2 = DisconnectFromMqm( );

      if ( rc2 > rc )
      {
         rc = rc2;
      }
   }

   /* Tell the user what we did. */

   if ( *(MR.PerformLogging) == 'Y' )
   {
      PrintStatistics( );
   }

   /* Time to clean up. */

   Terminate( );

   return(0);
}

/**
 * ==================================================================
 *
 * Function Name
 *  Initialize
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This function will initialize the various fields.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  MR  - clear global structure
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  Initialize( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int               rc=0;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering Initialize\n")

   /**
    * Initialize the rest of the global variables.
    */
   memset(&MR, '\0', sizeof(MRTR_GLOBAL_T));
#if defined(MVS)
   strcpy( MR.iniFile, "DD:MRTRIN" );
#elif defined(OS400)
   strcpy( MR.iniFile, "QIBM/UserData/capitalware/MRTR/mrtr.ini" );
#else
   strcpy( MR.iniFile, "mrtr.ini" );
#endif

   return rc;
}

/**
 * ==================================================================
 *
 * Function Name
 *  Terminate
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This function will free or de-allocate various fields.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void Terminate( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering Terminate\n")

   if ( MR.pIMB != NULL )
      free( MR.pIMB );

   if ( MR.pOMB != NULL )
      free( MR.pOMB );

   return;
}


/**
 * ==================================================================
 *
 * Function Name
 *  GetCmdLineParms
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This function will parse the inputted command line and
 *  extract the keywords and their values.
 *
 * Input parameters
 *  argc - The number of parameters
 *  argv - An array of keywords and values.
 *
 * Output
 *  MR.iniFile    - set ini filename
 *  MR.LogFile    - set log filename
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  GetCmdLineParms( int argc, char *argv[] )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   #define MaxKeyParams 3

   char    *Parmkey[MaxKeyParams] = { "-i", "-I", "-?"};
   int      i,j;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering GetCmdLineParms\n")

   if (argc > 0)
   {
      for (i=1; i < argc; i++)
      {
         for (j=0; j < MaxKeyParams; j++)
         {
            if (!strncmp( argv[i], Parmkey[j], 2))
            {
                break; /* fall out to switch stmt */
            }
         }

         switch (j)
         {
            case 0:                      /* -i */
            case 1:                      /* -I */
              if (i+1 < argc)
              {
                  strncpy( MR.iniFile, argv[i+1], _MAX_PATH );
                  i++;
              }
              else
              {
                 printf( "Error with keyword -i value\n" );
                 printf( "Use %s -? for usage\n", argv[0]);
                 return 12;
              }
              break;

            case 2:                      /* -? */
               printf( "\n%s %s\n", argv[0], usage);
               return 12;
               break;

            default:/* parameter error */
               printf( "Error with keyword parameter!\n" );
               printf( "Invalid flag.  Use %s -? for usage\n",argv[0]);
               return 12;
               break;
         }
      }
   }
   else
   {
      printf( "\nMissing parameters!\n\n" );
      printf( "%s %s\n", argv[0], usage);
      return 12;
   }


   return 0;
}


/**
 * ==================================================================
 *
 * Function Name
 *  GetInitFileParameters
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This function reads an initialization file for input parameters.
 *
 * Input parameters
 *  MR   - Pointer to the Global MRTR work area.
 *
 * Output
 *  MR.SequenceNumberFlag - Will the ini file contain sequence num
 *  MR.LogFile            - Path and name of the log file
 *  MR.QMgrName           - set field
 *  MR.InputQName         - set field
 *  MR.RouterDeadQName    - set field
 *  MR.TimeOut            - Set MQGET timeout value
 *  MR.MessageBufferSize  - max. buffer size
 *  MR.StartKeyword       - Set field
 *  MR.EndKeyword         - Set field
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  GetInitFileParameters( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int          rc = 0;
   int          Length;
   char         TempString[512];
   char         iniBuffer[MAX_INI_LINE_LENGTH];
   char         Buffer[3] = {'\0', '\0', '\0'};

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering GetInitFileParameters\n")

   TEST_DATA(("inifile=%s\n", MR.iniFile))

   /**
    * First get info about sequence numbers from ini file.
    */
   Length = GetProfileString( INI_SEQUENCE_NUMBER_FLAG,
                              DEFAULT_SEQUENCE_NUMBER_FLAG,
                              TempString,
                              72,
                              MR.iniFile,
                              'Y' );

   strcpy( Buffer, TempString);   /* move flag */

   /**
    * Did we find it?  If not found, assume "N"
    */
   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_SEQUENCE_NUMBER_FLAG );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );

      MR.SequenceNumberFlag = 'N';
   }
   else if ( ( *(Buffer) == 'y' ) || ( *(Buffer) == 'Y' ) )
   {
      MR.SequenceNumberFlag = 'Y';  /* uppercase it */
   }
   else if ( ( *(Buffer) == 'n' ) || ( *(Buffer) == 'N' ) )
   {
      MR.SequenceNumberFlag = 'N';  /* lowercase it */
   }
   else
   {
      MR.SequenceNumberFlag = 'N'; /* everything else make it No */
   }

   /**
    * Retrieve logfile info from ini file.
    */
   Length = GetProfileString( INI_LOGFILE,
                              DEFAULT_LOGFILE,
                              iniBuffer,
                              sizeof(iniBuffer),
                              MR.iniFile,
                              MR.SequenceNumberFlag );

   if ( Length == 0 )
   {                                   /* not found, tell the world. */
      printf( "Error: LogFile was not found.\n" );
      printf( "Specify the LogFile name in the configuration file\n");
      printf( "Using default value: %s\n", DEFAULT_LOGFILE );

      /**
       * Set a default value for the logfile
       */
      #ifdef MVS
      sprintf( MR.LogFile, "DD:%s", DEFAULT_LOGFILE );
      #else
      strcpy( MR.LogFile, DEFAULT_LOGFILE );
      #endif
   }
   else
   {
      #ifdef MVS
      int  k;

      /**
       * Count the length of logfile name
       */
      for (k = 0; k < _MAX_PATH; k++)
      {
         if ( iniBuffer[k] == '\0' )
         {
            break;
         }
      }

      /**
       * Check the length of logfile name
       */
      if (k > 8)
      {
         printf( "Error: Log file DD '%s' is too long.\n",
                 iniBuffer );

         /**
          * We will use just the first 8 characters
          */
         sprintf( MR.LogFile, "DD:%8.8s", iniBuffer );
      }
      else
      {
         sprintf( MR.LogFile, "DD:%s", iniBuffer );
      }

      #else
      strcpy( MR.LogFile, iniBuffer );
      #endif
   }

   /**
    * Get info about performing the logging.
    */
   Length = GetProfileString( INI_PERFORM_LOGGING,
                              DEFAULT_PERFORM_LOGGING,
                              TempString,
                              72,
                              MR.iniFile,
                              MR.SequenceNumberFlag );

   strcpy( MR.PerformLogging, TempString);   /* move flag */

   /**
    * Did we find it?  If not found, assume "N"
    */
   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_PERFORM_LOGGING );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );

      *(MR.PerformLogging) = 'N';
   }
   else if ( *(MR.PerformLogging) == 'y' )
   {
      *(MR.PerformLogging) = 'Y';  /* uppercase it */
   }
   else if ( ( *(MR.PerformLogging) != 'Y' ) &&
             ( *(MR.PerformLogging) != 'N' )  )
   {
      *(MR.PerformLogging) = 'N'; /* everything else make it No */
   }

   /**
    * Mark the begining of the log file.
    */
   if ( *(MR.PerformLogging) == 'Y' )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             "********************************************");

      LogIt( LOG_TYPE_INFO,
             __LINE__,
             "MRTR is now starting.");

      sprintf(TempString,
              "%s version %s (%s)",
              PROGRAM_NAME,
              EYECATCHER_Version,
              HOSTED_PLATFORM_NAME);
      LogIt( LOG_TYPE_INFO,  __LINE__, TempString );

      sprintf( TempString, "Current Log file is %s", MR.LogFile );
      LogIt( LOG_TYPE_INFO,  __LINE__, TempString );

      sprintf( TempString, "PerformLogging=%s", MR.PerformLogging);
      LogIt( LOG_TYPE_INFO,  __LINE__, TempString );
   }

   /**
    * Get all integer / binary values from the ini file.
    */

   /**
    * Retrieve the Time out value from ini file.
    */
   MR.TimeOut = GetProfileInt( INI_TIMEOUT,
                                DEFAULT_TIMEOUT_VALUE,
                                MR.iniFile,
                                MR.SequenceNumberFlag );

   /**
    * Retrieve the Message Buffer Size
    */
   MR.MessageBufferSize = GetProfileInt( INI_MSG_BUFFER_SIZE,
                                          DEFAULT_MSG_BUFFER_SIZE,
                                          MR.iniFile,
                                          MR.SequenceNumberFlag );

   /**
    * Get all string values from the ini file.
    */

   /**
    * Retrieve the Queue Manager Name
    */
   Length = GetProfileString( INI_QMGR_NAME,
                              DEFAULT_QMGR_NAME,
                              MR.QMgrName,
                              sizeof(MR.QMgrName),
                              MR.iniFile,
                              MR.SequenceNumberFlag );

   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_QMGR_NAME );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );
   }

   /**
    * Retrieve the Input Queue Name
    */
   Length = GetProfileString( INI_INPUT_Q_NAME,
                              DEFAULT_INPUT_Q_NAME,
                              MR.InputQName,
                              sizeof(MR.InputQName),
                              MR.iniFile,
                              MR.SequenceNumberFlag );

   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_INPUT_Q_NAME );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );
   }

   /**
    * Retrieve the IA Dead Queue Name
    */
   Length = GetProfileString( INI_DEAD_LETTER_QUEUE,
                              DEFAULT_DEAD_LETTER_QUEUE,
                              MR.RouterDeadQName,
                              sizeof(MR.RouterDeadQName),
                              MR.iniFile,
                              MR.SequenceNumberFlag );

   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_DEAD_LETTER_QUEUE );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );
   }

   /**
    * Retrieve the StartKeyword field
    */
   Length = GetProfileString( INI_START_KEYWORD,
                              DEFAULT_START_KEYWORD,
                              MR.StartKeyword,
                              sizeof(MR.StartKeyword),
                              MR.iniFile,
                              MR.SequenceNumberFlag );

   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_START_KEYWORD );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );

      rc = 12;
   }

   /**
    * Retrieve the EndKeyword field
    */
   Length = GetProfileString( INI_END_KEYWORD,
                              DEFAULT_END_KEYWORD,
                              MR.EndKeyword,
                              sizeof(MR.EndKeyword),
                              MR.iniFile,
                              MR.SequenceNumberFlag );

   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_END_KEYWORD );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );

      rc = 12;
   }

   /**
    * Retrieve the Get With Convert setting
    */
   Length = GetProfileString( INI_GET_WITH_CONVERT,
                              DEFAULT_INI_YES,
                              TempString,
                              72,
                              MR.iniFile,
                              MR.SequenceNumberFlag );

   strcpy( MR.GetWithConvert, TempString);   /* move flag */

   if ( Length == 0 )
   {
      sprintf( TempString,
               "%s has no value",
               INI_GET_WITH_CONVERT );
      LogIt( LOG_TYPE_WARNING,  __LINE__, TempString );

      *(MR.GetWithConvert) = 'N';
   }
   else if ( *(MR.GetWithConvert) == 'y' )
   {
      *(MR.GetWithConvert) = 'Y';  /* uppercase it */
   }
   else if ( ( *(MR.GetWithConvert) != 'Y' ) &&
             ( *(MR.GetWithConvert) != 'N' )  )
   {
      *(MR.GetWithConvert) = 'Y';
   }

   /**
    * Write all values to the log file??
    */
   if ( *(MR.PerformLogging) == 'Y' )
   {
      Print_MRTR_Global( );
   }

   return rc;
}


/**
 * ==================================================================
 *
 * Function Name
 *  Print_MRTR_Global
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This function will dump the global area.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void Print_MRTR_Global( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   char            Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering Print_MRTR_Global\n")

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          "The following are the parameters used for this execution.");

   /* Output the GetWithConvert flag */

   sprintf( MR.TempBuffer,
            "   GetWithConvert    : %c",
            *(MR.GetWithConvert) );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   /* Output Queue Manager name */

   memcpy( Buffer,
           MR.QMgrName,
           sizeof(MR.QMgrName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MR.TempBuffer,
            "   Queue Manager Name: %s",
            Buffer);

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   /* Output the Input Message Q name */

   memcpy( Buffer,
           MR.InputQName,
           sizeof(MR.InputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MR.TempBuffer,
            "   Input Queue Name  : %s",
            Buffer );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   /* Output the IA Dead Letter Q name */


   memcpy( Buffer,
           MR.RouterDeadQName,
           sizeof(MR.RouterDeadQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MR.TempBuffer,
            "   IA Dead Queue Name: %s",
            Buffer );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   /* Output the Start keyword */

   sprintf( MR.TempBuffer,
            "   StartKeyword      : %s",
            MR.StartKeyword );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   /* Output the Start keyword */

   sprintf( MR.TempBuffer,
            "   EndKeyword        : %s",
            MR.EndKeyword );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   /* Output the TimeOut value */

   sprintf( MR.TempBuffer,
            "   TimeOut value     : %d",
            MR.TimeOut );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   /* Output the Message Buffer Size */

   sprintf( MR.TempBuffer,
            "   Get Bufffer Size  : %d",
            MR.MessageBufferSize );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   return;
}


/**
 * ==================================================================
 *
 * Function Name
 *  PrintStatistics
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This function will print statistic information about what was done.
 *
 * Input parameters
 *  None.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void PrintStatistics( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering PrintStatistics\n")

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          "Statistics of this executions:" );

   /* Output MQGET count */

   sprintf( MR.TempBuffer,
            "   Messages Read     : %d",
            MR.TotalMsgRead );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   /* Output MQPUT count */

   sprintf( MR.TempBuffer,
            "   Messages Written  : %d",
            MR.TotalMsgWritten );

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          MR.TempBuffer );

   /* Tell the world we are going away. */

   LogIt( LOG_TYPE_INFO,
          __LINE__,
          "MRTR is now stopping.");

   return;
}


/**
 * ==================================================================
 *
 * Function Name
 *  LogIt
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This function will write a record to the logfile.
 *
 * Input parameters
 *  severity- The level of severity to be associated with this msg.
 *  line    - Line number of the calling function.
 *  msg     - The message text to be outputted.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void LogIt( int         severity,
                   int         line,
                   char       *msg )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   char            LogRecord[64];
   char            cSeverity[10];
   FILE           *LogFileHandle;
   struct tm*      CurrentTime;
   time_t          LongTime;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
/*   TEST_PRINT("Now entering LogIt\n") */

   /**
    * Open up the log file
    */
   if ( *(MR.LogFile) == '\0' )
   {
      LogFileHandle = stdout;              /* write to the console */
   }
   else
   {
      LogFileHandle = fopen( MR.LogFile, "a" ); /* write to a file */

      if ( LogFileHandle == NULL )
      {
         printf( "Failed to open the log file.\n" );
         return;
      }
   }

   /**
    * Change severity level to text description.
    */
   switch( severity )
   {
      case LOG_TYPE_INFO:
       strcpy( cSeverity, "INFO   " );
       break;
      case 1:
       strcpy( cSeverity, "WARNING" );
       break;
      case 2:
       strcpy( cSeverity, "ERROR  " );
       break;
      default:
       strcpy( cSeverity, "ERROR+ " );
       break;
    }

   /**
    * Get the current (local) time.
    */
   LongTime = time(NULL);
   CurrentTime = localtime(&LongTime);

   sprintf( LogRecord,
            LOG_RECORD_FORMAT_STRING,
            (CurrentTime->tm_year + 1900),
            (CurrentTime->tm_mon + 1),
            CurrentTime->tm_mday,
            CurrentTime->tm_hour,
            CurrentTime->tm_min,
            CurrentTime->tm_sec,
            cSeverity,
            line );

   /**
    * Write it out and close the file.
    */
   fputs( LogRecord, LogFileHandle );

   /**
    * Write the contents of the buffer.
    */
   fputs( msg, LogFileHandle );

   /**
    * Put a new line character after the data.
    */
   fputs( "\n", LogFileHandle );
   fflush( LogFileHandle );

   if ( *(MR.LogFile) != '\0' )
      fclose( LogFileHandle );

   return;
}

/**
 * ==================================================================
 *
 * Function Name
 *  RemoveTrailingBlanks
 *
 * Description
 *  This routine will remove trailing blanks from a string.
 *
 * Input parameters
 *  string - a pointer to a string where trailing blanks are to be
 *           removed - if any.
 *
 * Output
 *  string - update string with trailing blanks removed.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void RemoveTrailingBlanks( char *String )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int   Length;           /* Length of String */

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   /**
    * Find the 1st blank beyond the last valid character.
    */
   for ( Length = (strlen( String ) - 1 ); Length >= 0 ; Length-- )
   {
       if ( *(String + Length) != ' ' )     /* look for a non-blank */
       {
            break;                 /* get out - we found a non-blank */
       }
   }

   /**
    * Set trailing blanks to nulls.
    */
   memset( String + Length + 1, '\0', (strlen(String) - Length) );

   return;
}

/**
 * ==================================================================
 *
 * Function Name
 *  RemovePrefixBlanks
 *
 * Description
 *  This routine will remove any prefix blanks from a string.
 *
 * Input parameters
 *  string - a pointer to a string where prefix blanks are to be
 *           removed - if any.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void RemovePrefixBlanks( char *String )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int           Length;           /* Length of String */

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   /**
    * Find the 1st nonblank character.
    */
   for ( Length = 0; Length <= (int) strlen( String ); Length++ )
   {
       if ( *(String + Length) != ' ' )   /* look for a non-blank */
       {
          break;                 /* get out - we found a non-blank */
       }
   }

   /**
    * Remove beginning blanks.
    */
   if (Length != 0)
   {
      memcpy( String, String + Length, (strlen(String) - Length ) );
      memset( String + (strlen(String) - Length ), '\0', Length );
   }

   return;
}

/**
 * ==================================================================
 *
 * Function Name
 *  ConnectToMqm
 *
 * Description
 *  This routine will establish a connection between
 *  us and the local MQM. If it fails, put out a message
 *  and return a non-zero return code.
 *
 * Input parameters
 *  MR.QMgrName - Queue Manager name
 *
 * Output
 *  MR.Hconn - Connection handle
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  ConnectToMqm( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;   /* return code from MQM */
   MQLONG    mqRC; /* reason code from MQM */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering ConnectToMqm\n")

   MQCONN( MR.QMgrName,         /* queue manager */
           &(MR.Hconn),         /* connection handle */
           &mqCC,               /* completion code */
           &mqRC);              /* reason code */

   rc = mqRC;

   memcpy( Buffer, MR.QMgrName, MQ_Q_MGR_NAME_LENGTH );

   Buffer[MQ_Q_MGR_NAME_LENGTH] = '\0';  /* make sure we have a null */
   RemoveTrailingBlanks( Buffer );

   sprintf( MR.TempBuffer,
            "MQConn CC=%d RC=%d on connection to %s",
            mqCC,
            mqRC,
            Buffer );

   if ( mqCC == MQCC_OK )
   {
      MR.ConnectedFlag =  1;

      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MR.TempBuffer );
   }
   else
   {
      MR.ConnectedFlag =  0;

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MR.TempBuffer );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  DisconnectFromMqm
 *
 * Description
 *  This routine will sever an existing connection between
 *  us and MQM. If it fails, put out a message and return
 *  a non-zero return code.
 *
 * Input parameters
 *  MR.Hconn - Connection handle
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  DisconnectFromMqm( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;   /* return code from MQM */
   MQLONG    mqRC;   /* reason code from MQM */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering DisconnectFromMqm\n")

   MQDISC( &MR.Hconn,        /* connection handle */
           &mqCC,            /* completion code */
           &mqRC);           /* reason code */

   rc = mqRC;

   MR.ConnectedFlag =  0;

   memcpy( Buffer, MR.QMgrName, MQ_Q_MGR_NAME_LENGTH );

   Buffer[MQ_Q_MGR_NAME_LENGTH] = '\0';  /* make sure we have a null */
   RemoveTrailingBlanks( Buffer );

   sprintf( MR.TempBuffer,
            "MQDisc  CC=%d RC=%d on disconnecting from %s",
            mqCC,
            mqRC,
            Buffer );

   if ( mqCC == MQCC_OK )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MR.TempBuffer );
   }
   else
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MR.TempBuffer );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  OpenInputQueue
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This routine will open the input queue. If it fails,
 *  put out a message and return a non-zero return code.
 *
 * Input parameters
 *  MR.Hconn      - Connection handle for the local Queue Manager
 *  MR.InputQName - Input Q name
 *
 * Output
 *  MR.HobjInQ - Object handle for Input Q
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  OpenInputQueue( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;       /* return code from MQM */
   MQLONG    mqRC;     /* reason code from MQM */
   MQOD      od  = {MQOD_DEFAULT};
   MQLONG    OpenOptions;       /* Open Options */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering OpenInputQueue\n")

   memcpy( od.ObjectQMgrName,
           MR.QMgrName,
           MQ_Q_MGR_NAME_LENGTH);

   memcpy( od.ObjectName,
           MR.InputQName,
           MQ_Q_NAME_LENGTH);

   OpenOptions = MQOO_FAIL_IF_QUIESCING + MQOO_INPUT_EXCLUSIVE +
                 MQOO_SAVE_ALL_CONTEXT;

   MQOPEN( MR.Hconn,              /* connection handle */
           &od,                   /* object descriptor for queue */
           OpenOptions,           /* open options */
           &(MR.HobjInQ),         /* object handle */
           &mqCC,                 /* completion code */
           &mqRC);                /* reason code */

   rc = mqRC;

  /* Output the Completion Code and Reason Code */

   memcpy( Buffer, MR.InputQName, sizeof(MR.InputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';
   RemoveTrailingBlanks( Buffer );

   sprintf( MR.TempBuffer,
            "MQOpen CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer);

   if ( mqCC == MQCC_OK )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MR.TempBuffer );

      MR.OpenedInputQueue = 1;
   }
   else
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MR.TempBuffer );

      MR.OpenedInputQueue = 0;
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  CloseInputQueue
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This routine will close the input queue. If it fails,
 *  put out a message and return a non-zero return code.
 *
 * Input parameters
 *  MR.Hconn   - Connection handle for the local Queue Manager
 *  MR.HobjInQ - Object handle for Input Q
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  CloseInputQueue( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int       rc=0;
   MQLONG    mqCC;       /* return code from MQM */
   MQLONG    mqRC;       /* reason code from MQM */
   char      Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering CloseInputQueue\n")

   MQCLOSE( MR.Hconn,
            &(MR.HobjInQ),
            MQCO_NONE,
            &mqCC,
            &mqRC);

   MR.OpenedInputQueue = 0;

  /* Output the Completion Code and Reason Code */

   memcpy( Buffer, MR.InputQName, sizeof(MR.InputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';
   RemoveTrailingBlanks( Buffer );

   sprintf( MR.TempBuffer,
            "MQClose CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer);

   if ( mqCC == MQCC_OK )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MR.TempBuffer );
   }
   else
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MR.TempBuffer );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  ProcessMessages
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This routine will browse the Input Q for any messages that
 *  have the particaular named field with the value that matches
 *  what the user wants.
 *
 * Input parameters
 *  MR.Hconn    - connection handle
 *  MR.HobjInQ  - Object handle for Input Q
 *  MR.HobjOutQ - Object handle for Output Q
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  ProcessMessages( void )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int         rc=0, rc2=0;
   char        Continue   = 'Y';

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering ProcessMessages\n")

   MR.pIMB = malloc(
                          CalculateMemorySize(MR.MessageBufferSize));

   MR.pOMB = malloc(
                          CalculateMemorySize(MR.MessageBufferSize));

   /* Did we allocate memory? */

   if ( (MR.pIMB == NULL) || (MR.pOMB == NULL) )
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             "*** Fatal Error: Cannot allocate MQGET message buffer "\
             "memory! ***" );
      rc = 12;
   }
   else
   {
      memset( MR.pIMB, '\0', MR.MessageBufferSize );
      memset( MR.pOMB, '\0', MR.MessageBufferSize );

      /**
       * Setup and get first message.
       */
      rc = ReadMessage( );

      if ( rc != MQRC_NONE )
      {
         Continue = 'N';
      }

      /**
       * Loop through the messages on the queue.
       */
      while ( (Continue == 'Y') && (rc == 0) )
      {
         FindKeyword();

         rc = PutMessage( MR.OutputQName );

         /* Problem - throw it on the appl dead letter q */
         if ( rc != 0 )
         {
            LogIt( LOG_TYPE_WARNING,
                   __LINE__,
                   "Attempting to write the message to the IA Dead "\
                   "Letter Queue." );

            rc = PutMessage( MR.RouterDeadQName );
         }

         if ( rc == 0 )
         {
            rc = CommitMessages();
         }
         else
         {
            rc2 = BackOutMessages();

            if ( rc2 > rc )
               rc = rc2;
         }

         if ( rc == 0 )
         {
            rc = ReadMessage();
         }

         if ( rc != MQRC_NONE )
         {
            Continue = 'N';
         }
      }
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  ReadMessage
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This routine will read (either browse or destructive get based on
 *  extra get options) a message from a queue.  If it fails, put
 *  out a message and return a non-zero return code.
 *
 * Input parameters
 *  ExtraOptions  - Extra get options either browse or destructive get
 *  MR.Hconn    - Connection handle for the local Queue Manager
 *  MR.HobjOutQ - Object handle for Output Q
 *
 * Output
 *  MR.pIMB - data returned from MQGET
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  ReadMessage( )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int           rc=0;
   MQLONG        mqCC;       /* return code from MQM */
   MQLONG        mqRC;       /* reason code from MQM */
   MQLONG        MessageLength = 0;
   MQGMO         gmo = {MQGMO_DEFAULT};
   MQMD          md  = {MQMD_DEFAULT};
   char          Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering ReadMessage\n")

   gmo.Options=MQGMO_FAIL_IF_QUIESCING | MQGMO_SYNCPOINT | MQGMO_WAIT;

   if ( *(MR.GetWithConvert) == 'Y' )
      gmo.Options = gmo.Options | MQGMO_CONVERT;

   if (MR.TimeOut == 0)
      gmo.WaitInterval = MQWI_UNLIMITED;
   else
      gmo.WaitInterval = MR.TimeOut * 1000;

   memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
   memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

   MQGET( MR.Hconn,                 /* connection handle */
          MR.HobjInQ,               /* object handle */
          &md,                      /* message descriptor */
          &gmo,                     /* get message options */
          MR.MessageBufferSize,     /* buffer length */
          MR.pIMB,                  /* message buffer */
          &MR.ActualMessageSize,    /* returned message length */
          &mqCC,                    /* completion code */
          &mqRC);                   /* reason code */

   rc = mqRC;

   memcpy( &(MR.OriginalMD), &(md), sizeof(MQMD) );

   /* Output the Completion Code and Reason Code */

   memcpy( Buffer,
           MR.InputQName,
           sizeof(MR.InputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MR.TempBuffer,
            "MQGet  CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer);

   /**
    * Did everything work?? i.e. returned a message
    */
   if ( (mqCC == MQCC_OK) && (mqRC == MQRC_NONE) )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MR.TempBuffer );

      if (md.Feedback == MQFB_QUIT)
      {
         LogIt( LOG_TYPE_INFO,
                __LINE__,
                "Quit message received - time to go." );

         rc = 5;
      }

      MR.TotalMsgRead++;
   }

   /**
    * No more messages??
    */
   else if ( (mqCC != MQCC_OK) &&
             (mqRC == MQRC_NO_MSG_AVAILABLE) )
   {
      /* No more messages on the queue. */
      LogIt( LOG_TYPE_WARNING,
             __LINE__,
             MR.TempBuffer );
   }

   else if ( (mqCC == MQCC_WARNING) &&
             (mqRC == MQRC_FORMAT_ERROR) )
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MR.TempBuffer );

      LogIt( LOG_TYPE_WARNING,
             __LINE__,
             "Use GetWithConvert=N to avoid this error." );

      BackOutMessages();
   }

   /**
    * Something catastrophic happen!!
    */
   else if ( mqCC != MQCC_OK )
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MR.TempBuffer );

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             "*** Severe Error accessing the queue. ***" );

      BackOutMessages();
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  FindKeyword
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This routine will find a keyword value between 2 keywords (start
 *  and end).  This keyword value will be used to lookup an inifile
 *  string that relates to an output queue name.
 *
 *  i.e. In the message:
 *                        <StartKeyword>ini-value<EndKeyword>
 *
 *       In the inifile:
 *                        ini-value = Output.Queue.Name
 *
 * Input parameters
 *  MR.pIMB - Input message buffer
 *
 * Output
 *  MR.OutputQName - Name of Output Q
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
void FindKeyword( )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int           Length=0;
   char         *beginning, *ending;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering FindKeyword\n")

   /* put a null behind the message. */

   memset( MR.pIMB + MR.ActualMessageSize, '\0', 1) ;

   memset( MR.KeywordValue, '\0', sizeof(MR.KeywordValue) );

   beginning = strstr( MR.pIMB, MR.StartKeyword );

   if ( beginning != NULL )
   {
      Length = strlen( MR.StartKeyword );

      ending = strstr( beginning + Length, MR.EndKeyword );

      Length =  ending - beginning - Length ;

      memcpy( MR.KeywordValue,
              beginning + strlen(MR.StartKeyword),
              Length );

      sprintf( MR.TempBuffer,
               "Found %s between %s and %s",
               MR.KeywordValue,
               MR.StartKeyword,
               MR.EndKeyword );
      LogIt( LOG_TYPE_INFO,  __LINE__, MR.TempBuffer );

      /**
       * Retrieve the KeywordValue -> Queue Name
       */
      Length = GetProfileString( MR.KeywordValue,
                                 "",
                                 MR.OutputQName,
                                 sizeof(MR.OutputQName),
                                 MR.iniFile,
                                 MR.SequenceNumberFlag );

      if ( Length == 0 )
      {
         sprintf( MR.TempBuffer,
                  "%s has no value",
                  MR.KeywordValue );
         LogIt( LOG_TYPE_WARNING,  __LINE__, MR.TempBuffer );

         memcpy( MR.OutputQName,
                 MR.RouterDeadQName,
                 MQ_Q_NAME_LENGTH );
      }
   }
   else
   {
      memcpy( MR.OutputQName, MR.RouterDeadQName, MQ_Q_NAME_LENGTH );
   }

   sprintf( MR.TempBuffer,
            "Output Queue will be %s",
            MR.OutputQName );
   LogIt( LOG_TYPE_INFO,  __LINE__, MR.TempBuffer );

   return;
}

/**
 * ==================================================================
 *
 * Function Name
 *  PutMessage
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This routine will write (put) a message to the output queue.
 *  If it fails, put out a message and return a non-zero return code.
 *
 * Input parameters
 *  OutputQName  - name of Output queue
 *  MR.Hconn    - Connection handle for the local Queue Manager
 *  MR.pIMB - outbound message buffer.
 *  MR.ActualMessageSize - size of the message.
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  PutMessage( MQCHAR  *OutputQName )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int          rc=0;
   MQLONG       mqCC;       /* return code from MQM */
   MQLONG       mqRC;       /* reason code from MQM */
   int          MessageLength = 0;
   MQOD         od  = {MQOD_DEFAULT};
   MQPMO        pmo = {MQPMO_DEFAULT};
   MQMD         md  = {MQMD_DEFAULT};
   char         Buffer[101];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering PutMessage\n")

   /**
    * Set object name
    */
   memset( od.ObjectQMgrName,
           ' ',
           MQ_Q_MGR_NAME_LENGTH);

   memcpy( od.ObjectName,
           OutputQName,
           MQ_Q_NAME_LENGTH);

   /**
    * Set PUT options
    */
   pmo.Options = MQPMO_SYNCPOINT + MQPMO_FAIL_IF_QUIESCING +
                 MQPMO_PASS_ALL_CONTEXT;

   /* pass in the Input Queue's handle */

   pmo.Context = MR.HobjInQ;

   /* move original MD values. */

   memcpy( &(md), &(MR.OriginalMD), sizeof(MQMD) );

   MQPUT1( MR.Hconn,               /* connection handle */
           &od,                    /* object descriptor */
           &md,                    /* message descriptor */
           &pmo,                   /* put message options */
           MR.ActualMessageSize,   /* buffer length */
           MR.pIMB,                /* message buffer */
           &mqCC,                  /* completion code */
           &mqRC);                 /* reason code */

   rc = mqRC;

  /* Output the Completion Code and Reason Code */
   memcpy( Buffer,
           OutputQName,
           sizeof(MR.OutputQName) );

   Buffer[MQ_Q_NAME_LENGTH] = '\0';

   RemoveTrailingBlanks( Buffer );

   sprintf( MR.TempBuffer,
            "MQPut1 CC=%d RC=%d for %s",
            mqCC,
            mqRC,
            Buffer );

   /**
    * Was the put1 successful??
    */
   if ( (mqCC == MQCC_OK) && (mqRC == MQRC_NONE) )
   {
      LogIt( LOG_TYPE_INFO, __LINE__, MR.TempBuffer );
      /* Life's good!!!! */
      MR.TotalMsgWritten++;
   }

   /**
    * Something catastrophic happen!! Again!!
    */
   else if ( mqCC != MQCC_OK )
   {
      LogIt( LOG_TYPE_ERROR, __LINE__, MR.TempBuffer );

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             "*** Severe Error accessing the queue. ***" );
   }


   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  CommitMessages
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This routine will commit the previous gets and puts.
 *
 * Input parameters
 *  MR.Hconn    - Connection handle for the local Queue Manager
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  CommitMessages( )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int           rc=0;
   MQLONG        mqCC;       /* return code from MQM */
   MQLONG        mqRC;       /* reason code from MQM */

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering CommitMessages\n")

   MQCMIT( MR.Hconn,                 /* connection handle */
           &mqCC,                    /* completion code */
           &mqRC);                   /* reason code */

   rc = mqRC;

   sprintf( MR.TempBuffer,
            "MQCmit CC=%d RC=%d",
            mqCC,
            mqRC );

   /**
    * Did everything work?? i.e. returned a message
    */
   if ( (mqCC == MQCC_OK) && (mqRC == MQRC_NONE) )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MR.TempBuffer );
   }

   /**
    * Something catastrophic happen!!
    */
   else if ( mqCC != MQCC_OK )
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MR.TempBuffer );

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             "*** Severe Error committing the messages. ***" );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  BackOutMessages
 *
 * Last date of modification
 *  18 Nov 2001
 *
 * Description
 *  This routine will rollback the previous gets and puts.
 *
 * Input parameters
 *  MR.Hconn    - Connection handle for the local Queue Manager
 *
 * Output
 *  None.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  BackOutMessages( )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   int           rc=0;
   MQLONG        mqCC;       /* return code from MQM */
   MQLONG        mqRC;       /* reason code from MQM */

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering BackOutMessages\n")

   MQBACK( MR.Hconn,                 /* connection handle */
           &mqCC,                    /* completion code */
           &mqRC);                   /* reason code */

   rc = mqRC;

   sprintf( MR.TempBuffer,
            "MQBack CC=%d RC=%d",
            mqCC,
            mqRC );

   /**
    * Did everything work?? i.e. returned a message
    */
   if ( (mqCC == MQCC_OK) && (mqRC == MQRC_NONE) )
   {
      LogIt( LOG_TYPE_INFO,
             __LINE__,
             MR.TempBuffer );
   }

   /**
    * Something catastrophic happen!!
    */
   else if ( mqCC != MQCC_OK )
   {
      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             MR.TempBuffer );

      LogIt( LOG_TYPE_ERROR,
             __LINE__,
             "*** Severe Error committing the messages. ***" );
   }

   return (rc);
}

/**
 * ==================================================================
 *
 * Function Name
 *  ReadLine
 *
 * Description
 *  This function will read a line from the specified file.  If the
 *  sequence number flag is on then truncate the line at 72 characters.
 *
 * Input parameters
 *  fp - a pointer to the file to be read from
 *  bp - a pointer to the copy buffer
 *  SeqNumFlag - MVS sequence ## in the file?
 *
 * Output
 *  None.
 *
 * Return Value
 *  TRUE if successful FALSE otherwise
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  ReadLine( FILE     *fp,
               char     *bp,
               char     SeqNumFlag )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
    char    c = '\0';
    int     i = 0;

    /* --------------------------------------------
     * Code section
     * --------------------------------------------
     */
/*   TEST_PRINT("Now entering ReadLine\n") */

   /**
    * Read one line at a time from the source file
    */
   if (!feof (fp) && fgets (bp, MAX_INI_LINE_LENGTH, fp) != 0)
   {
      if ( strchr (bp, '\r') )
      {
          *strchr (bp, '\r') = '\0';
      }
      else if ( strchr (bp, '\n') )
      {
          *strchr (bp, '\n') = '\0';
      }
      else
      {
         /**
          * Make sure the line is at a length that we can handle.
          */
         memset( (bp+MAX_INI_LINE_LENGTH-1), 0 , 1 );
      }

      /**
       * If the sequence number flag is set then truncate the line
       * at column 72.
       */
      if ( (SeqNumFlag == 'y') || (SeqNumFlag == 'Y') )
      {
         bp[71] = '\0';   /* sequence numbers begin in column 72 */
      }

      return(1);   /* Yes!! we have a line. */
   }

   return(0);   /* !! no line. */
}

/**
 * ==================================================================
 *
 * Function Name
 *  GetProfileInt
 *
 * Description
 *  This function will retrieve an integer from an initialization
 *  file based on the provided field "entry" name.
 *
 * Input parameters
 *  entry - the name of the entry to find the value of
 *  def - the default value in the event of a failed read
 *  file_name - the name of the .ini file to read from
 *  SeqNumFlag - MVS sequence ## in the file?
 *
 * Output
 *  None.
 *
 * Return Value
 *  the value located at entry
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int GetProfileInt( char   *entry,
                   int     def,
                   char   *file_name,
                   char    SeqNumFlag)
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   FILE           *fp;
   int             len;
   int             i;
   char           *ep;
   char            buff[MAX_INI_LINE_LENGTH] = {'\0'};
   char            value[12];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering GetProfileInt\n")

   memset( buff, '\0', MAX_INI_LINE_LENGTH );

   len = strlen(entry);

   /**
    * Open up the ini file.
    */
   if ( !(fp = fopen(file_name,"r")) )
   {
      return( def );
   }

   /**
    * Move through file 1 line at a time until a the entry
    * is matched or EOF. '[' is section header - end if found.
    */
   do
   {
       if( !ReadLine( fp, buff, SeqNumFlag ) ||
            buff[0] == '\0' ||
            buff[0] == '[')
       {
           fclose( fp );
           return( def );
       }
   }  while( strncmp( buff, entry, len) );

   ep = strrchr(buff,'=');    /* Parse out the equal sign */
   ep++;

   if( !strlen(ep) )          /* No setting? */
   {
       return(def);
   }

   /**
    * Remove unnecessary blanks
    */
   RemovePrefixBlanks( ep );
   RemoveTrailingBlanks( ep );

   /**
    * Copy only numbers - fail on characters
    */
   for(i = 0; isdigit(ep[i]); i++ )
   {
       value[i] = ep[i];
   }

   value[i] = '\0';
   fclose(fp);                /* Clean up and return the value */

   return( atoi(value) );
}

/**
 * ==================================================================
 *
 * Function Name
 *  GetProfileString
 *
 * Description
 *  This function will retrieve a string from an initialization
 *  file based on the provided field "entry" name.
 *
 * Input parameters
 *  section - the name of the section to search for
 *  entry - the name of the entry to find the value of
 *  def - default string in the event of a failed read
 *  buffer - a pointer to the buffer to copy into
 *  buffer_len - the max number of characters to copy
 *  file_name - the name of the .ini file to read from
 *  SeqNumFlag - MVS sequence ## in the file?
 *
 * Output
 *  None.
 *
 * Return Value
 *  number of characters copied into the supplied buffer
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int GetProfileString( char   *entry,
                      char   *def,
                      char   *buffer,
                      int     buffer_len,
                      char   *file_name,
                      char    SeqNumFlag)
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   FILE           *fp;
   int             len = strlen(entry);
   char           *ep;
   char            buff[MAX_INI_LINE_LENGTH] = {'\0'};

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering GetProfileString\n")

   memset( buff, '\0', MAX_INI_LINE_LENGTH );

   /**
    * Open up the ini file.
    */
   if ( !(fp = fopen(file_name,"r")) )
   {
        strncpy(buffer,def,buffer_len);
        return(strlen(buffer));
   }

   /**
    * Move through file 1 line at a time until a the entry
    * is matched or EOF. '[' is section header - end if found.
    */
   do
   {
      if( !ReadLine(fp, buff, SeqNumFlag) ||
           buff[0] == '\0' ||
           buff[0] == '[' )
       {
           fclose(fp);
           strncpy(buffer,def,buffer_len);
           return(strlen(buffer));
       }
   }  while( strncmp(buff,entry,len) );

   ep = strrchr(buff,'=');    /* Parse out the equal sign */
   ep++;

   /**
    * Copy up to buffer_len chars to buffer
    */
   strncpy(buffer,ep,(buffer_len - 1) );

   /**
    * Remove unnecessary blanks
    */
   RemovePrefixBlanks( buffer );
   RemoveTrailingBlanks( buffer );

   /**
    * Clean up and return the amount copied
    */
   fclose(fp);

   return( strlen(buffer) );
}

/**
 * ==================================================================
 *
 * Function Name
 *  CalculateMemorySize
 *
 * Description
 *  This routine will calculate the required memory size. It
 *  rounds the number up to the nearest 4 KB boundary.
 *
 * Input parameters
 *  Length - Input length to be used in the base calculation.
 *
 * Output
 *  The length rounded to the nearest 4 KB boundary.
 *
 * Return Value
 *  None.
 *
 * Abend Codes
 *  None.
 *
 * Called Routines
 *  None.
 *
 * ------------------------------------------------------------------
 */
int  CalculateMemorySize( int Length )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   TEST_PRINT("Now entering CalculateMemorySize\n")

   return ( ((Length + 1023) / 1024) * 1024 );
}

/* End of file */
